/*
  Copyright <2018-2025> <scott.e.graves@protonmail.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#ifndef LIBMONITARR_INCLUDE_ACTIONS_HPP_
#define LIBMONITARR_INCLUDE_ACTIONS_HPP_

#include "utils/config.hpp"

namespace monitarr {
struct app_config;
struct server_cfg;
class data_db;

[[nodiscard]] auto create_client(const server_cfg &server) -> httplib::Client;

[[nodiscard]] auto get_download(std::uint64_t record_id,
                                const server_cfg &server,
                                bool *is_not_found = nullptr)
    -> std::optional<nlohmann::json>;

[[nodiscard]] auto list_queue(const server_cfg &server) -> int;

[[nodiscard]] auto load_config(std::string &cfg_file) -> app_config;

[[nodiscard]] auto load_db(std::string_view log_dir) -> data_db;

void remove_stalled(std::string_view download_id, std::string_view title,
                    std::uint64_t episode_id, std::uint64_t movie_id,
                    const server_cfg &server, data_db *state_db = nullptr);

[[nodiscard]] auto test_server(const server_cfg &server) -> int;
} // namespace monitarr

#endif // LIBMONITARR_INCLUDE_ACTIONS_HPP_
