/*
  Copyright <2018-2025> <scott.e.graves@protonmail.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#include "args.hpp"

namespace monitarr {
auto get_arg(std::string_view arg, int argc, char **argv)
    -> std::optional<std::string> {
  auto cmd_line = std::span(argv, static_cast<std::size_t>(argc)) |
                  std::views::transform(
                      [](char const *val) { return std::string_view(val); });
  auto iter =
      std::ranges::find_if(std::next(cmd_line.begin()), cmd_line.end(),
                           [&arg](auto &&item) -> bool { return arg == item; });
  if (iter == cmd_line.end()) {
    return std::nullopt;
  }

  iter = std::next(iter);
  if (iter == cmd_line.end()) {
    return std::nullopt;
  }

  return std::string{*iter};
}

auto has_arg(std::string_view arg, int argc, char **argv) -> bool {
  auto cmd_line = std::span(argv, static_cast<std::size_t>(argc)) |
                  std::views::transform(
                      [](char const *val) { return std::string_view(val); });
  return std::ranges::find_if(std::next(cmd_line.begin()), cmd_line.end(),
                              [&arg](auto &&item) -> bool {
                                return arg == item;
                              }) != cmd_line.end();
}
} // namespace monitarr
